#include <msp/datafile/builtinsource.h>
#include <msp/fs/utils.h>
#include <msp/gl/extensions/sgis_generate_mipmap.h>
#include "animation.h"
#include "armature.h"
#include "font.h"
#include "keyframe.h"
#include "material.h"
#include "mesh.h"
#include "object.h"
#include "pose.h"
#include "program.h"
#include "programcompiler.h"
#include "resourcemanager.h"
#include "resources.h"
#include "technique.h"
#include "texture1d.h"
#include "texture2d.h"
#include "texture2darray.h"
#include "texturecube.h"

using namespace std;

namespace Msp {
namespace GL {

void init_shaderlib(DataFile::BuiltinSource &);

Resources::Resources():
	default_tex_filter(SGIS_generate_mipmap ? LINEAR_MIPMAP_LINEAR : LINEAR),
	srgb_conversion(false),
	resource_manager(0)
{
	add_type<Animation>().suffix(".anim").keyword("animation");
	add_type<Armature>().suffix(".arma").keyword("armature");
	add_type<Font>().keyword("font");
	add_type<KeyFrame>().suffix(".kframe").keyword("keyframe");
	add_type<Material>().suffix(".mat").keyword("material");
	add_type<Mesh>().keyword("mesh").creator(&Resources::create_mesh);
	add_type<Object>().keyword("object");
	add_type<Pose>().keyword("pose");
	add_type<Program>().keyword("shader").suffix(".glsl").creator(&Resources::create_program);
	add_type<Technique>().suffix(".tech").keyword("technique");
	add_type<Texture1D>().base<Texture>().suffix(".tex1d").keyword("texture1d");
	add_type<Texture2D>().base<Texture>().suffix(".tex2d").suffix(".png").suffix(".jpg").keyword("texture2d").creator(&Resources::create_texture2d);
	add_type<Texture3D>().base<Texture>().suffix(".tex3d").keyword("texture3d");
	add_type<TextureCube>().base<Texture>().suffix(".texcb").keyword("texture_cube");
	add_type<Texture2DArray>().base<Texture>().suffix(".tex2da").keyword("texture2d_array");

	add_source(get_builtins());
}

const DataFile::CollectionSource &Resources::get_builtins()
{
	static DataFile::BuiltinSource builtins;
	bool init_done = false;

	if(!init_done)
	{
		init_shaderlib(builtins);
		init_done = true;
	}

	return builtins;
}

void Resources::set_default_texture_filter(TextureFilter tf)
{
	default_tex_filter = tf;
}

void Resources::set_srgb_conversion(bool c)
{
	srgb_conversion = c;
}

void Resources::set_resource_manager(ResourceManager *m)
{
	resource_manager = m;
}

Mesh *Resources::create_mesh(const string &name)
{
	if(!resource_manager)
		return 0;

	if(RefPtr<IO::Seekable> io = open_from_sources(name))
	{
		RefPtr<GL::Mesh> mesh = new GL::Mesh(resource_manager);
		resource_manager->set_resource_location(*mesh, *this, name);
		return mesh.release();
	}

	return 0;
}

Texture2D *Resources::create_texture2d(const string &name)
{
	string ext = FS::extpart(name);
	if(ext==".tex2d")
		return 0;

	if(RefPtr<IO::Seekable> io = open_from_sources(name))
	{
		Graphics::Image image;
		if(!resource_manager)
			image.load_io(*io);

		RefPtr<GL::Texture2D> tex = new GL::Texture2D(resource_manager);

		if(is_mipmapped(default_tex_filter))
		{
			tex->set_generate_mipmap(true);
			tex->set_mag_filter(LINEAR);
		}
		else
			tex->set_mag_filter(default_tex_filter);
		tex->set_min_filter(default_tex_filter);

		if(resource_manager)
			resource_manager->set_resource_location(*tex, *this, name);
		else
			tex->image(image, srgb_conversion);
		return tex.release();
	}

	return 0;
}

Program *Resources::create_program(const string &name)
{
	string ext = FS::extpart(name);
	if(ext==".shader")
		return 0;

	if(RefPtr<IO::Seekable> io = open_from_sources(name))
	{
		ProgramCompiler compiler;
		compiler.compile(*io, this);
		RefPtr<Program> program = new Program;
		compiler.add_shaders(*program);
		program->link();
		return program.release();
	}

	return 0;
}

} // namespace GL
} // namespace Msp
